@@ -220,10 +220,17 @@ void execute_packet_child(
        the path to the mtr-packet executable.  This is necessary
        for debugging changes for mtr-packet.
      */
-    char *mtr_packet_path = getenv("MTR_PACKET");
-    if (mtr_packet_path == NULL) {
+    char * mtr_packet_path = NULL; 
+
+    // In the rare case that mtr-packet is not setuid-root, 
+    // and a select group of users has sudo privileges to run 
+    // mtr and not much else, THEN create /etc/mtr.is.run.under.sudo
+    // to prevent a privilege escalation when one of those accounts
+    // is compromised.  CVE-2025-49809
+    if (access ("/etc/mtr.is.run.under.sudo", F_OK) != 0)
+        mtr_packet_path = getenv("MTR_PACKET");
+    if (mtr_packet_path == NULL)
         mtr_packet_path = "mtr-packet";
-    }
 
     /*
        First, try to execute mtr-packet from PATH
@@ -417,6 +424,22 @@ void append_command_argument(
     strncat(command, argument, remaining_size);
 }
 
+static
+void append_command_string_argument(
+    char *command,
+    int buffer_size,
+    char *name,
+    char *value)
+{
+    char argument[COMMAND_BUFFER_SIZE];
+    int remaining_size;
+
+    remaining_size = buffer_size - strlen(command) - 1;
+
+    snprintf(argument, buffer_size, " %s %s", name, value);
+    strncat(command, argument, remaining_size);
+}
+
 
 /*  Request a new probe from the "mtr-packet" child process  */
 void send_probe_command(
@@ -466,6 +489,11 @@ void send_probe_command(
     }
 #endif
 
+    if (ctl->InterfaceName) {
+        append_command_string_argument(command, COMMAND_BUFFER_SIZE,
+                                       "local-device", ctl->InterfaceName);
+    }
+
     remaining_size = COMMAND_BUFFER_SIZE - strlen(command) - 1;
     strncat(command, "\n", remaining_size);
 
@@ -694,10 +722,14 @@ void handle_command_reply(
     if (!strcmp(reply_name, "reply")
             || !strcmp(reply_name, "ttl-expired")) {
         err = 0;
-    } else if (!strcmp(reply_name, "no-route")) {
-        err = ENETUNREACH;
     } else if (!strcmp(reply_name, "network-down")) {
         err = ENETDOWN;
+    } else if (!strcmp(reply_name, "host-down")) {
+        err = EHOSTDOWN;
+    } else if (!strcmp(reply_name, "no-route-network")) {
+        err = ENETUNREACH;
+    } else if (!strcmp(reply_name, "no-route-host")) {
+        err = EHOSTUNREACH;
     } else {
         /*  If the reply type is unknown, ignore it  */
         return;
